package core.api.models.timemanager.taskSchedule;

import core.api.models.timemanager.taskgroup.TaskgroupEntityInfo;
import core.api.models.timemanager.tasks.TaskShortInfo;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.ScheduleType;
import core.entities.timemanager.Taskgroup;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

public class ScheduleInfo {

    private long scheduleID;
    private ScheduleType scheduleType;
    private LocalDateTime startTime;

    private LocalDateTime finishTime;
    private int activeMinutes;
    private AbstractScheduleShortInfo schedule;

    private List<TaskgroupEntityInfo> taskgroupPath;

    private TaskShortInfo task;

    public ScheduleInfo(BasicTaskSchedule basicTaskSchedule) {
        this.scheduleID = basicTaskSchedule.getScheduleID();
        this.scheduleType = ScheduleType.BASIC;
        this.startTime = basicTaskSchedule.getStartTime();
        this.finishTime = basicTaskSchedule.getFinishedTime();

        if(this.finishTime == null && this.startTime != null) {
            this.activeMinutes = (int) Duration.between(basicTaskSchedule.getStartTime(), LocalDate.now()).toMinutes();
        } else if(this.startTime != null){
            this.activeMinutes = (int) Duration.between(basicTaskSchedule.getStartTime(), basicTaskSchedule.getFinishedTime()).toMinutes();
        }

        this.schedule = new BasicScheduleShortInfo(basicTaskSchedule);
        this.task = new TaskShortInfo(basicTaskSchedule.getTask());
        List<Taskgroup> taskgroupPath = Taskgroup.getAncestorList(basicTaskSchedule.getTask().getTaskgroup());
        taskgroupPath.add(basicTaskSchedule.getTask().getTaskgroup());
        this.taskgroupPath = taskgroupPath.stream().map(TaskgroupEntityInfo::new).toList();
    }

    public long getScheduleID() {
        return scheduleID;
    }

    public void setScheduleID(long scheduleID) {
        this.scheduleID = scheduleID;
    }

    public ScheduleType getScheduleType() {
        return scheduleType;
    }

    public void setScheduleType(ScheduleType scheduleType) {
        this.scheduleType = scheduleType;
    }

    public LocalDateTime getStartTime() {
        return startTime;
    }

    public void setStartTime(LocalDateTime startTime) {
        this.startTime = startTime;
    }

    public LocalDateTime getFinishTime() {
        return finishTime;
    }

    public void setFinishTime(LocalDateTime finishTime) {
        this.finishTime = finishTime;
    }

    public int getActiveMinutes() {
        return activeMinutes;
    }

    public void setActiveMinutes(int activeMinutes) {
        this.activeMinutes = activeMinutes;
    }

    public AbstractScheduleShortInfo getSchedule() {
        return schedule;
    }

    public void setSchedule(AbstractScheduleShortInfo schedule) {
        this.schedule = schedule;
    }

    public List<TaskgroupEntityInfo> getTaskgroupPath() {
        return taskgroupPath;
    }

    public void setTaskgroupPath(List<TaskgroupEntityInfo> taskgroupPath) {
        this.taskgroupPath = taskgroupPath;
    }

    public TaskShortInfo getTask() {
        return task;
    }

    public void setTask(TaskShortInfo task) {
        this.task = task;
    }
}
