package core.entities.timemanager;

import javax.persistence.*;
import java.time.LocalDate;
import java.util.Objects;
import java.util.Set;

@Entity
@Table(name = "tasks")
public class Task {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private long taskID;

    @ManyToOne
    @JoinColumn(name = "taskgroup_id")
    private Taskgroup taskgroup;
    private String taskName;

    private LocalDate startDate;

    private LocalDate deadline;

    private int eta;

    private boolean finished;

    @OneToMany(mappedBy = "task", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    private Set<BasicTaskSchedule> basicTaskSchedules;

    private int workTime;

    public Task(Taskgroup taskgroup, String taskName, LocalDate startDate, LocalDate deadline, int eta) {
        this.taskgroup = taskgroup;
        this.taskName = taskName;
        this.startDate = startDate;
        this.deadline = deadline;
        this.eta = eta;
        this.finished = false;
    }

    public Task() {
    }

    public long getTaskID() {
        return taskID;
    }

    public Taskgroup getTaskgroup() {
        return taskgroup;
    }

    public void setTaskgroup(Taskgroup taskgroup) {
        this.taskgroup = taskgroup;
    }

    public String getTaskName() {
        return taskName;
    }

    public void setTaskName(String taskName) {
        this.taskName = taskName;
    }

    public LocalDate getStartDate() {
        return startDate;
    }

    public void setStartDate(LocalDate startDate) {
        this.startDate = startDate;
    }

    public LocalDate getDeadline() {
        return deadline;
    }

    public void setDeadline(LocalDate deadline) {
        this.deadline = deadline;
    }

    public int getEta() {
        return eta;
    }

    public void setEta(int eta) {
        this.eta = eta;
    }

    public void finish() {
        this.finished = true;
    }

    public boolean isFinished() {
        return finished;
    }

    public void setFinished(boolean finished) {
        this.finished = finished;
    }

    public int getWorkTime() {
        return workTime;
    }

    public void setWorkTime(int workTime) {
        this.workTime = workTime;
    }

    public void addWorkTime(int workTime) {
        this.workTime += workTime;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Task task = (Task) o;
        return taskID == task.taskID;
    }

    @Override
    public int hashCode() {
        return Objects.hash(taskID);
    }

    public void setTaskID(long taskID) {
        this.taskID = taskID;
    }

    public Set<BasicTaskSchedule> getBasicTaskSchedules() {
        return basicTaskSchedules;
    }

    public void setBasicTaskSchedules(Set<BasicTaskSchedule> basicTaskSchedules) {
        this.basicTaskSchedules = basicTaskSchedules;
    }
}
