package core.services;

import core.api.models.timemanager.taskSchedule.BasicTaskScheduleFieldInfo;
import core.entities.User;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.Task;
import core.repositories.UserRepository;
import core.repositories.timemanager.BasicTaskScheduleRepository;
import core.repositories.timemanager.TaskRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

@Service
public class TaskScheduleService {

    private final BasicTaskScheduleRepository basicTaskScheduleRepository;
    private final TaskRepository taskRepository;
    private final UserRepository userRepository;

    public TaskScheduleService(@Autowired BasicTaskScheduleRepository basicTaskScheduleRepository,
                               @Autowired TaskRepository taskRepository,
                               UserRepository userRepository) {
        this.basicTaskScheduleRepository = basicTaskScheduleRepository;
        this.taskRepository = taskRepository;
        this.userRepository = userRepository;
    }

    public ServiceResult<BasicTaskSchedule> createBasicTaskSchedule(Task task, BasicTaskScheduleFieldInfo basicTaskScheduleFieldInfo) {
        BasicTaskSchedule basicTaskSchedule = new BasicTaskSchedule(task, basicTaskScheduleFieldInfo.getScheduleDate());
        task.getBasicTaskSchedules().add(basicTaskSchedule);
        basicTaskScheduleRepository.save(basicTaskSchedule);
        taskRepository.save(task);
        return new ServiceResult<>(basicTaskSchedule);
    }

    public PermissionResult<BasicTaskSchedule> getSchedulePermissions(long scheduleID, String name) {
        Optional<BasicTaskSchedule> basicSchedule = basicTaskScheduleRepository.findById(scheduleID);
        return basicSchedule.map(value -> new PermissionResult<>(value, value.getTask().getTaskgroup().getUser().getUsername().equals(name))).orElseGet(() ->
                new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public void reschedule(BasicTaskSchedule basicTaskSchedule, BasicTaskScheduleFieldInfo basicTaskScheduleFieldInfo) {
        basicTaskSchedule.setScheduleDate(basicTaskScheduleFieldInfo.getScheduleDate());
        basicTaskScheduleRepository.save(basicTaskSchedule);
    }

    public void deleteBasicSchedule(BasicTaskSchedule basicTaskSchedule) {
        basicTaskScheduleRepository.deleteBasicTaskScheduleByID(basicTaskSchedule.getScheduleID());
    }

    public ServiceResult<List<BasicTaskSchedule>> loadTodaysSchedule(String username) {
        Optional<User> user = userRepository.findByUsername(username);
        return user.map(value -> new ServiceResult<>(basicTaskScheduleRepository.findAllByUserAndDate(value, LocalDate.now()))).orElseGet(() ->
                new ServiceResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<List<BasicTaskSchedule>> loadSchedules(String username) {
        Optional<User> user = userRepository.findByUsername(username);
        return user.map(value -> new ServiceResult<>(basicTaskScheduleRepository.findAllByUser(value))).orElseGet(() ->
                new ServiceResult<>(ServiceExitCode.MISSING_ENTITY));
    }
}
