package core.services;

import core.api.models.timemanager.tasks.TaskFieldInfo;
import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import core.repositories.timemanager.TaskRepository;
import core.repositories.timemanager.TaskgroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Optional;

@Service
public class TaskService {

    private final TaskRepository taskRepository;
    private final TaskgroupRepository taskgroupRepository;

    public TaskService(@Autowired TaskRepository taskRepository,
                       TaskgroupRepository taskgroupRepository) {
        this.taskRepository = taskRepository;
        this.taskgroupRepository = taskgroupRepository;
    }

    public ServiceResult<Task> createTask(Taskgroup taskgroup, TaskFieldInfo taskFieldInfo) {
        if(existTaskByName(taskgroup.getTasks(), taskFieldInfo.getTaskName())) {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        }


        Task task = new Task(taskgroup, taskFieldInfo.getTaskName(), taskFieldInfo.getStartDate(), taskFieldInfo.getDeadline(), taskFieldInfo.getEta());
        taskgroup.getTasks().add(task);
        taskRepository.save(task);

        return new ServiceResult<>(task);
    }

    private boolean existTaskByName(Collection<Task> tasks, String name) {
        for(Task task : tasks) {
            if(task.getTaskName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    public PermissionResult<Task> getTaskPermissions(long taskID, String name) {
        Optional<Task> task = taskRepository.findById(taskID);
        return task.map(value -> new PermissionResult<>(value, value.getTaskgroup().getUser().getUsername().equals(name))).orElseGet(() ->
                new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<Task> editTask(Task task, TaskFieldInfo taskFieldInfo) {
        if(!task.getTaskName().equals(taskFieldInfo.getTaskName()) && !existTaskByName(task.getTaskgroup().getTasks(), taskFieldInfo.getTaskName())) {
            task.setTaskName(taskFieldInfo.getTaskName());

        } else if(!task.getTaskName().equals(taskFieldInfo.getTaskName()) && existTaskByName(task.getTaskgroup().getTasks(), taskFieldInfo.getTaskName())) {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        }
        task.setEta(taskFieldInfo.getEta());
        task.setStartDate(taskFieldInfo.getStartDate());
        task.setDeadline(taskFieldInfo.getDeadline());
        taskRepository.save(task);
        return new ServiceResult<>(task);
    }

    public void deleteTask(Task task) {
        taskRepository.deleteByTaskID(task.getTaskID());
    }

    public void clearTasks(Taskgroup taskgroup) {
        taskRepository.deleteAllByTaskgroup(taskgroup);
    }
}
