import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges} from '@angular/core';
import {FormControl, Validators} from "@angular/forms";
import {BasicScheduleEntityInfo, ScheduleService, TaskEntityInfo, TaskgroupEntityInfo} from "../../../api";
import {MatSnackBar} from "@angular/material/snack-bar";
import {Router} from "@angular/router";

@Component({
  selector: 'app-basic-scheduler',
  templateUrl: './basic-scheduler.component.html',
  styleUrls: ['./basic-scheduler.component.css']
})
export class BasicSchedulerComponent implements OnChanges{

  dateCtrl: FormControl = new FormControl('', [Validators.required])
  @Input('taskgroup') taskgroup: TaskgroupEntityInfo | undefined
  @Input('task') task: TaskEntityInfo | undefined
  @Input('scheduleInfo') scheduleEntityInfo: BasicScheduleEntityInfo | undefined
  @Output('onSchedule') scheduleEmitter: EventEmitter<BasicScheduleEntityInfo> = new EventEmitter<BasicScheduleEntityInfo>();

  constructor(private scheduleService: ScheduleService,
              private snackbar: MatSnackBar,
              private router: Router) {
  }

  setDate(date: Date) {
    this.dateCtrl.setValue(date);
  }

  schedule() {
    if(this.task != undefined) {
      if(this.scheduleEntityInfo == undefined) {
        this.scheduleService.schedulesTaskIDBasicPut(this.task.taskID, {
          scheduleDate: this.dateCtrl.value
        }).subscribe({
          next: resp => {
            this.scheduleEmitter.emit(resp);
          }
        })
      } else {
        this.scheduleService.schedulesScheduleIDBasicPost(this.scheduleEntityInfo!.scheduleID, {
          scheduleDate: this.dateCtrl.value
        }).subscribe({
          next: resp => {
            this.router.navigateByUrl("/taskgroups/" + this.taskgroup!.taskgroupID + "/tasks/" + this.task!.taskID);
          },
          error: err => {
            if(err.status == 403) {
              this.snackbar.open("No permission", "",{duration: 2000});
            } else if(err.status == 404) {
              this.snackbar.open("Not found", "", {duration: 2000});
            } else {
              this.snackbar.open("Unexpected error", "",{duration: 2000});
            }
          }
        })
      }

    }
  }

  ngOnInit(): void {
    if(this.scheduleEntityInfo != undefined) {
      this.dateCtrl.setValue(this.scheduleEntityInfo!.scheduleID)
    }
  }

  ngOnChanges(): void {
    if(this.scheduleEntityInfo != undefined) {
      this.dateCtrl.setValue(this.scheduleEntityInfo!.scheduleID)
    }
  }

  setEditedBasicSchedule(basicSchedule: BasicScheduleEntityInfo) {
    this.dateCtrl.setValue(basicSchedule.scheduleDate)
    this.scheduleEntityInfo = basicSchedule;
    console.log(this.dateCtrl.value)
  }
}
