import {Component, Input, OnInit} from '@angular/core';
import {BasicScheduleEntityInfo, ScheduleService, TaskEntityInfo, TaskgroupEntityInfo} from "../../../api";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'app-schedule-dashboard',
  templateUrl: './schedule-dashboard.component.html',
  styleUrls: ['./schedule-dashboard.component.css']
})
export class ScheduleDashboardComponent implements OnInit{

  @Input('taskgroup')  taskgroup: TaskgroupEntityInfo | undefined
  @Input('task') task: TaskEntityInfo | undefined

  schedules: BasicScheduleEntityInfo[] = []

  constructor(private scheduleService: ScheduleService,
              private snackbar: MatSnackBar) {

  }

  ngOnInit() {
    this.scheduleService.schedulesTaskIDScheduleTypeGet(this.task!.taskID, "BASIC").subscribe({
      next: resp => {
        this.schedules = resp;
      }
    })
  }

  reschedule() {
    //todo
  }

  deleteSchedule(deletedSchedule: BasicScheduleEntityInfo) {
    this.scheduleService.schedulesScheduleIDScheduleTypeDelete(deletedSchedule.scheduleID, 'BASIC').subscribe({
      next: resp => {
        this.schedules = this.schedules.filter(schedule => schedule.scheduleID !== deletedSchedule.scheduleID);
      },
      error: err => {
        if(err.status == 403) {
          this.snackbar.open("No permission", "", {duration: 2000});
        } else if(err.status == 404) {
          this.snackbar.open("Schedule not found", "", {duration: 2000});
        } else {
          this.snackbar.open("Unexpected error", "",{duration: 2000});
        }
      }
    })
  }
}
