import {Component, OnInit, ViewChild} from '@angular/core';
import {NavigationLink, NavigationLinkListComponent} from "../../navigation-link-list/navigation-link-list.component";
import {ActivatedRoute} from "@angular/router";
import {TaskEntityInfo, TaskgroupEntityInfo, TaskgroupService, TaskService} from "../../../api";
import {TaskDashboardComponent} from "../task-dashboard/task-dashboard.component";

@Component({
  selector: 'app-task-detail-overview',
  templateUrl: './task-detail-overview.component.html',
  styleUrls: ['./task-detail-overview.component.css']
})
export class TaskDetailOverviewComponent implements OnInit {

  defaultNavigationLinkPath: NavigationLink[] = [
    {
      linkText: "Dashboard",
      routerLink: ['/']
    },
    {
      linkText: "Taskgroups",
      routerLink: ["/taskgroups"]
    }
  ]
  taskgroups: TaskgroupEntityInfo[] = []
  taskgroup: TaskgroupEntityInfo | undefined
  taskgroupPath: TaskgroupEntityInfo[] = []
  taskgroupID: number = -1;
  @ViewChild('navLinkList') navLinkListComponent: NavigationLinkListComponent | undefined

  task: TaskEntityInfo | undefined

  constructor(private activatedRoute: ActivatedRoute,
              private taskgroupService: TaskgroupService,
              private taskService: TaskService) {
  }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      if (params.has('taskgroupID')) {
        this.taskgroupID = Number(params.get('taskgroupID'));
        this.taskgroupService.taskgroupsTaskgroupIDGet(this.taskgroupID).subscribe({
          next: resp => {
            this.taskgroups = resp.children
            this.taskgroupPath = resp.ancestors
            this.taskgroup = resp.taskgroupInfo;
            this.navLinkListComponent!.addNavigationLink(this.taskgroup.taskgroupName, ['/taskgroups', this.taskgroup.taskgroupID.toString()])
            this.taskgroupPath.forEach(taskgroupEntity => {
              this.navLinkListComponent!.addNavigationLink(taskgroupEntity.taskgroupName, ['/taskgroups', taskgroupEntity.taskgroupID.toString()]);
            })
          }
        })
      }

      if(params.has('taskID')) {
        this.taskService.tasksTaskIDGet(Number(params.get('taskID'))).subscribe({
          next: resp => {
            this.task = resp;
          }
        })
      }
    });
  }

  getStatusOfTask(task: TaskEntityInfo | undefined) {
    return "green";

  }
}
